<?php
/*
Plugin Name: Jambopay Payment
Plugin URI: https://deveint.com/en
Description: Checkout woocommerce using jambopay gateway
Version: 1.0.0
Author: Enock Kibet
Author URI: https://enockkibet.com
Text Domain: Deveint

*/

if ( ! defined( 'ABSPATH' ) ) {
	exit;
}


/*
 * This action hook registers our PHP class as a WooCommerce payment gateway
 */
add_filter( 'woocommerce_payment_gateways', 'jambopay_add_gateway_class' );
function jambopay_add_gateway_class( $gateways ) {
	$gateways[] = 'WC_Jambopay_Gateway'; // your class name is here

	return $gateways;
}

add_action( 'plugins_loaded', 'jambopay_init_gateway_class' );
function jambopay_init_gateway_class() {

	class WC_Jambopay_Gateway extends WC_Payment_Gateway {

		/**
		 * Class constructor, more about it in Step 3
		 */
		public function __construct() {

			$this->id                 = 'jambopay'; // payment gateway plugin ID
			$this->icon               = ''; // URL of the icon that will be displayed on checkout page near your gateway name
			$this->has_fields         = true; // in case you need a custom credit card form
			$this->method_title       = 'Jambopay Gateway';
			$this->method_description = 'Access several mobile and card payment options using our gateway.'; // will be displayed on the options page

			// gateways can support subscriptions, refunds, saved payment methods,
			// but in this tutorial we begin with simple payments
			$this->supports = array(
				'products'
			);

			// Method with all the options fields
			$this->init_form_fields();

			// Load the settings.
			$this->init_settings();
			$this->title          = $this->get_option('title');
			$this->description    = $this->get_option('description');
			$this->enabled        = $this->get_option('enabled');
			$this->testmode       = 'yes' === $this->get_option('testmode');
			$this->business_email = $this->get_option('business_email');
			$this->phone_number   = $this->get_option('phone_number');
			$this->client_key     = $this->get_option('client_key');
			$this->checksum_key     = $this->get_option('checksum_key');
			$this->support_email = $this->get_option('support_phone');


			// This action hook saves the settings
			add_action( 'woocommerce_update_options_payment_gateways_' . $this->id, array(
				$this,
				'process_admin_options'
			) );

			// We need custom JavaScript to create the checkout
			add_action( 'wp_enqueue_scripts', array( $this, 'payment_scripts' ) );

			add_action( 'woocommerce_receipt_' . $this->id, array( &$this, 'payment_page' ) );

			// You can also register a webhook here
			add_action( 'woocommerce_api_jpcallback', array( $this, 'webhook' ) );
		}


		public function init_form_fields() {

			$this->form_fields = array(
				'enabled'        => array(
					'title'       => 'Enable/Disable',
					'label'       => 'Enable Jambopay Gateway',
					'type'        => 'checkbox',
					'description' => '',
					'default'     => 'no'
				),
				'title'          => array(
					'title'       => 'Title',
					'type'        => 'text',
					'description' => 'This controls the title (Store Name) which the user sees during checkout.',
					'default'     => '',
					'desc_tip'    => true,
				),
				'description'    => array(
					'title'       => 'Description',
					'type'        => 'textarea',
					'description' => 'This controls the description which the user sees during checkout.',
					'default'     => 'Pay with your credit card via our super-cool payment gateway.',
				),
				'testmode'       => array(
					'title'       => 'Test mode',
					'label'       => 'Enable Test Mode',
					'type'        => 'checkbox',
					'description' => 'Place the payment gateway in test mode using test API keys.',
					'default'     => 'yes',
					'desc_tip'    => true,
				),
				'business_email' => array(
					'title' => 'Business Email',
					'type'  => 'text'
				),
				/*'jp_password'    => array(
					'title'       => 'Password',
					'type'        => 'password',
					'description' => 'Password used to login to jambopay backend',
				),*/
				'client_key'     => array(
					'title' => 'Client Key',
					'type'  => 'text'
				),
				'checksum_key'     => array(
					'title' => 'Checksum Key',
					'type'  => 'text'
				),
				'phone_number'   => array(
					'title' => 'Support Phone Number',
					'type'  => 'text'
				),
				'support_email'   => array(
					'title' => 'Support Email Address',
					'type'  => 'text'
				)
			);

		}

		public function payment_scripts() {

			global $woocommerce;

			// we need JavaScript to process a token only on cart/checkout pages, right?
			if ( ! is_cart() && ! is_checkout() && ! isset( $_GET['pay_for_order'] ) ) {
				return;
			}

			// if our payment gateway is disabled, we do not have to enqueue JS too
			if ( 'no' === $this->enabled ) {
				return;
			}

			// no reason to enqueue JavaScript if API keys are not set
			if ( empty( $this->business_email ) || empty($this->client_key) || empty($this->checksum_key)) {
				return;
			}

			// do not work with card details without SSL unless your website is in a test mode
			if ( ! $this->testmode && ! is_ssl() ) {
				return;
			}

		}

		// /*
		//  * We're processing the payments here, everything about it is in Step 5
		//  */
		public function process_payment( $order_id ) {

			global $woocommerce;

			$order = wc_get_order( $order_id );

			if ( $order->get_status() === 'completed' || $order->get_status() === 'processing' ) {
				return array(
					'result'   => 'success',
					'redirect' => $this->get_return_url( $order )
				);
			} else {
				return array(
					'result'   => 'success',
					'redirect' => $order->get_checkout_payment_url( true )
				);
			}

		}

		public function payment_page( $order_id ) {

			$order = wc_get_order( $order_id );

			$client_key = $this->client_key;
			$support_phone  = $this->phone_number;
			$support_email = $this->support_email;
			$business_email = $this->business_email;

			//no more need for token

			if ( ! isset( $_POST['Status'] ) ) {
				if ( $this->testmode ) {
					wp_enqueue_script( 'jambopay', 'https://checkout-v3-test.jambopay.co.ke/sdk/checkout-staging-sdk.js',array() , null);
				} else {
					wp_enqueue_script( 'jambopay', 'https://checkoutv3.jambopay.com/sdk/checkout-sdk.js',array() , null);
				}

				wp_enqueue_script( 'bootstrap', 'https://cdnjs.cloudflare.com/ajax/libs/twitter-bootstrap/4.5.3/js/bootstrap.bundle.min.js');

				wp_register_script( 'jp_d_js', WP_PLUGIN_URL . '/woo_jambopay/assets/js/jp-checkout-v3.js', array( 'jquery' ) );

				wp_localize_script(
					'jp_d_js',
					'jp_ajax',
					array(
						'ajaxurl'        => admin_url('admin-ajax.php'),
						'ajaxnonce'      => wp_create_nonce('ajax_post_validation'),
						'customer_email' => $order->get_billing_email(),
						'customer_phone' => $order->get_billing_phone(),
						'business_email' => $business_email,
						'client_key'     => $client_key,
						'phone_number'   => $support_phone,
						'support_email'  => $support_email,
						'totals'         => $order->get_total(),
						'description'    => 'Payment for order id: '.$order_id,
						'currency'       => get_woocommerce_currency(),
						'orderid'        => $order_id,
						'checksum'       => $this->calculateStartChecksum($order_id,$order->get_total()),
						'cancelledurl'   => wc_get_checkout_url(),
						'callbackurl'    => $order->get_checkout_payment_url(true),
					)
				);

				wp_enqueue_script( 'jp_d_js' );

			}
			// Generate jp tokens

			if ( isset( $_POST['status'] ) ) {
				$status = $_POST['status'];
				$orderId = $_POST['order_id'];
				$wcOrder = wc_get_order($orderId);
				$orderStatus = $wcOrder->get_status();
				if($orderStatus!='pending') return;
				if(!$this->validateCallbackChecksum($wcOrder->get_id(),$wcOrder->get_total(),$_POST['checksum'])) return;
			} else {
				return;
			}
			if ( $status == 'SUCCESS' ) {
				$order->update_status( 'processing', 'Payment Completed' );
				$order->payment_complete();
			} else {
				$order->update_status( 'pending', 'Pending Payment.' );
				$order->add_order_note( __( '<strong>Order Pending</strong><br> Pending Payment' ) );
			}

			wp_redirect( $this->get_return_url( $order ), 301 );
		}

		// webhook
		public function jpcallback() {

			$order = wc_get_order( $_GET['id'] );
			$order->payment_complete();
			$order->reduce_order_stock();

			update_option( 'webhook_debug', $_GET );

		}

		private function calculateStartChecksum($order_id,$amount){
			return hash('sha256',number_format($amount,2,'.','').$this->checksum_key.$order_id);
		}

		private function validateCallbackChecksum($order_id,$amount,$checksum){
			return hash('sha256',$order_id.number_format($amount,2,'.','').$this->checksum_key)===$checksum;
		}
	}
}
?>